@@ -184,7 +184,7 @@ static bool split;
 static bool uniform;
 
 /* Prefix minus leading and trailing spaces (default "").  */
-static const unsigned char *prefix;
+static const char *prefix;
 
 /* User-supplied maximum line width (default WIDTH).  The only output
    lines longer than this will each comprise a single word.  */
@@ -273,19 +273,28 @@ usage (int status)
 Reformat each paragraph in the FILE(s), writing to standard output.\n\
 If no FILE or if FILE is `-', read standard input.\n\
 \n\
+"), stdout);
+      fputs (_("\
 Mandatory arguments to long options are mandatory for short options too.\n\
+"), stdout);
+      fputs (_("\
   -c, --crown-margin        preserve indentation of first two lines\n\
   -p, --prefix=STRING       combine only lines having STRING as prefix\n\
   -s, --split-only          split long lines, but do not refill\n\
+"),
+	     stdout);
+      fputs (_("\
   -t, --tagged-paragraph    indentation of first line different from second\n\
   -u, --uniform-spacing     one space between words, two after sentences\n\
   -w, --width=NUMBER        maximum line width (default of 75 columns)\n\
-      --help                display this help and exit\n\
-      --version             output version information and exit\n\
+"), stdout);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      fputs (_("\
 \n\
 In -wNUMBER, the letter `w' may be omitted.\n"),
 	     stdout);
-      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -568,7 +577,7 @@ copy_rest (FILE *f, register int c)
     {
       put_space (next_prefix_indent);
       for (s = prefix; out_column != in_column && *s; out_column++)
-	putchar (*s++);
+	putchar (*(unsigned char *)s++);
       put_space (in_column - out_column);
     }
   while (c != '\n' && c != EOF)
@@ -653,7 +662,6 @@ static int
 get_prefix (FILE *f)
 {
   register int c;
-  register const unsigned char *p;
 
   in_column = 0;
   c = get_space (f, getc (f));
@@ -662,10 +670,11 @@ get_prefix (FILE *f)
       prefix_lead_space : in_column;
   else
     {
+      const char *p;
       next_prefix_indent = in_column;
       for (p = prefix; *p != '\0'; p++)
 	{
-	  if (c != *p)
+	  if (c != *(unsigned char *)p)
 	    return c;
 	  in_column++;
 	  c = getc (f);
