@@ -1,5 +1,5 @@
 /* head -- output first part of file(s)
-   Copyright (C) 89, 90, 91, 1995-2001 Free Software Foundation, Inc.
+   Copyright (C) 89, 90, 91, 1995-2002 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include "system.h"
 #include "closeout.h"
 #include "error.h"
+#include "posixver.h"
 #include "xstrtol.h"
 #include "safe-read.h"
 
@@ -55,6 +56,9 @@ enum header_mode
   multiple_files, always, never
 };
 
+/* Options corresponding to header_mode values.  */
+static char const header_mode_option[][4] = { "", " -v", " -q" };
+
 /* The name this program was run with. */
 char *program_name;
 
@@ -90,21 +94,25 @@ Print first 10 lines of each FILE to standard output.\n\
 With more than one FILE, precede each with a header giving the file name.\n\
 With no FILE, or when FILE is -, read standard input.\n\
 \n\
+"), stdout);
+      fputs (_("\
 Mandatory arguments to long options are mandatory for short options too.\n\
+"), stdout);
+      fputs (_("\
   -c, --bytes=SIZE         print first SIZE bytes\n\
   -n, --lines=NUMBER       print first NUMBER lines instead of first 10\n\
 "), stdout);
       fputs (_("\
   -q, --quiet, --silent    never print headers giving file names\n\
   -v, --verbose            always print headers giving file names\n\
-      --help               display this help and exit\n\
-      --version            output version information and exit\n\
+"), stdout);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      fputs (_("\
 \n\
 SIZE may have a multiplier suffix: b for 512, k for 1K, m for 1 Meg.\n\
-If -VALUE is used as first OPTION, read -c VALUE when one of\n\
-multipliers bkm follows concatenated, else read -n VALUE.\n\
 "), stdout);
-      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -169,7 +177,20 @@ head_lines (const char *filename, int fd, uintmax_t lines_to_write)
 	break;
       while (bytes_to_write < bytes_read)
 	if (buffer[bytes_to_write++] == '\n' && --lines_to_write == 0)
-	  break;
+	  {
+	    /* If we have read more data than that on the specified number
+	       of lines, try to seek back to the position we would have
+	       gotten to had we been reading one byte at a time.  */
+	    if (lseek (fd, bytes_to_write - bytes_read, SEEK_CUR) < 0)
+	      {
+		int e = errno;
+		struct stat st;
+		if (fstat (fd, &st) != 0 || S_ISREG (st.st_mode))
+		  error (0, e, _("cannot reposition file pointer for %s"),
+			 filename);
+	      }
+	    break;
+	  }
       if (fwrite (buffer, 1, bytes_to_write, stdout) == 0)
 	error (EXIT_FAILURE, errno, _("write error"));
     }
@@ -251,7 +272,6 @@ main (int argc, char **argv)
 {
   enum header_mode header_mode = multiple_files;
   int exit_status = 0;
-  char *n_string;
   int c;
 
   /* Number of items to print. */
@@ -272,27 +292,25 @@ main (int argc, char **argv)
 
   print_headers = 0;
 
-  if (argc > 1 && argv[1][0] == '-' && ISDIGIT (argv[1][1]))
+  if (1 < argc && argv[1][0] == '-' && ISDIGIT (argv[1][1]))
     {
+      char *a = argv[1];
+      char *n_string = ++a;
       char *end_n_string;
       char multiplier_char = 0;
 
-      n_string = &argv[1][1];
-
       /* Old option syntax; a dash, one or more digits, and one or
 	 more option letters.  Move past the number. */
-      for (++argv[1]; ISDIGIT (*argv[1]); ++argv[1])
-	{
-	  /* empty */
-	}
+      do ++a;
+      while (ISDIGIT (*a));
 
       /* Pointer to the byte after the last digit.  */
-      end_n_string = argv[1];
+      end_n_string = a;
 
       /* Parse any appended option letters. */
-      while (*argv[1])
+      for (; *a; a++)
 	{
-	  switch (*argv[1])
+	  switch (*a)
 	    {
 	    case 'c':
 	      count_lines = 0;
@@ -303,7 +321,7 @@ main (int argc, char **argv)
 	    case 'k':
 	    case 'm':
 	      count_lines = 0;
-	      multiplier_char = *argv[1];
+	      multiplier_char = *a;
 	      break;
 
 	    case 'l':
@@ -319,10 +337,19 @@ main (int argc, char **argv)
 	      break;
 
 	    default:
-	      error (0, 0, _("unrecognized option `-%c'"), *argv[1]);
+	      error (0, 0, _("unrecognized option `-%c'"), *a);
 	      usage (1);
 	    }
-	  ++argv[1];
+	}
+
+      if (200112 <= posix2_version ())
+	{
+	  error (0, 0, _("`-%s' option is obsolete; use `-%c %.*s%.*s%s'"),
+		 n_string, count_lines ? 'n' : 'c',
+		 (int) (end_n_string - n_string), n_string,
+		 multiplier_char != 0, &multiplier_char,
+		 header_mode_option[header_mode]);
+	  usage (EXIT_FAILURE);
 	}
 
       /* Append the multiplier character (if any) onto the end of
