@@ -1,5 +1,5 @@
 /* nl -- number lines of files
-   Copyright (C) 89, 92, 1995-1999 Free Software Foundation, Inc.
+   Copyright (C) 89, 92, 1995-2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <getopt.h>
 
 #include "system.h"
+#include "closeout.h"
 
 #include <regex.h>
 
@@ -35,7 +36,7 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "nl"
 
-#define AUTHORS "Scott Bartram and David MacKenzie"
+#define AUTHORS N_ ("Scott Bartram and David MacKenzie")
 
 #ifndef TRUE
 # define TRUE   1
@@ -176,27 +177,40 @@ usage (int status)
 Usage: %s [OPTION]... [FILE]...\n\
 "),
 	      program_name);
-      printf (_("\
+      fputs (_("\
 Write each FILE to standard output, with line numbers added.\n\
 With no FILE, or when FILE is -, read standard input.\n\
 \n\
+"), stdout);
+      fputs (_("\
+Mandatory arguments to long options are mandatory for short options too.\n\
+"), stdout);
+      fputs (_("\
   -b, --body-numbering=STYLE      use STYLE for numbering body lines\n\
   -d, --section-delimiter=CC      use CC for separating logical pages\n\
   -f, --footer-numbering=STYLE    use STYLE for numbering footer lines\n\
+"), stdout);
+      fputs (_("\
   -h, --header-numbering=STYLE    use STYLE for numbering header lines\n\
   -i, --page-increment=NUMBER     line number increment at each line\n\
   -l, --join-blank-lines=NUMBER   group of NUMBER empty lines counted as one\n\
   -n, --number-format=FORMAT      insert line numbers according to FORMAT\n\
   -p, --no-renumber               do not reset line numbers at logical pages\n\
   -s, --number-separator=STRING   add STRING after (possible) line number\n\
+"), stdout);
+      fputs (_("\
   -v, --first-page=NUMBER         first line number on each logical page\n\
   -w, --number-width=NUMBER       use NUMBER columns for line numbers\n\
-      --help                      display this help and exit\n\
-      --version                   output version information and exit\n\
+"), stdout);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      fputs (_("\
 \n\
 By default, selects -v1 -i1 -l1 -sTAB -w6 -nrn -hn -bt -fn.  CC are\n\
 two delimiter characters for separating logical pages, a missing\n\
 second character implies :.  Type \\\\ for \\.  STYLE is one of:\n\
+"), stdout);
+      fputs (_("\
 \n\
   a         number all lines\n\
   t         number only nonempty lines\n\
@@ -209,8 +223,8 @@ FORMAT is one of:\n\
   rn   right justified, no leading zeros\n\
   rz   right justified, leading zeros\n\
 \n\
-"));
-      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
+"), stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -331,7 +345,7 @@ proc_text (void)
 	      blank_lines = 0;
 	    }
 	  else
-	    printf (print_no_line_fmt);
+	    puts (print_no_line_fmt);
 	}
       else
 	print_lineno ();
@@ -340,15 +354,15 @@ proc_text (void)
       if (1 < line_buf.length)
 	print_lineno ();
       else
-	printf (print_no_line_fmt);
+	puts (print_no_line_fmt);
       break;
     case 'n':
-      printf (print_no_line_fmt);
+      puts (print_no_line_fmt);
       break;
     case 'p':
       if (re_search (current_regex, line_buf.buffer, line_buf.length - 1,
 		     0, line_buf.length - 1, (struct re_registers *) 0) < 0)
-	printf (print_no_line_fmt);
+	puts (print_no_line_fmt);
       else
 	print_lineno ();
       break;
@@ -362,7 +376,7 @@ static enum section
 check_section (void)
 {
   size_t len = line_buf.length - 1;
-  
+
   if (len < 2 || memcmp (line_buf.buffer, section_del, 2))
     return Text;
   if (len == header_del_len
@@ -453,6 +467,8 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+  atexit (close_stdout);
+
   have_read_stdin = 0;
 
   while ((c = getopt_long (argc, argv, "h:b:f:v:i:pl:s:w:n:d:", longopts,
@@ -604,8 +620,6 @@ main (int argc, char **argv)
       error (0, errno, "-");
       exit_status = 1;
     }
-  if (ferror (stdout) || fclose (stdout) == EOF)
-    error (EXIT_FAILURE, errno, _("write error"));
 
   exit (exit_status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
