@@ -1,5 +1,5 @@
 /* split.c -- split a file into pieces.
-   Copyright (C) 88, 91, 1995-2001 Free Software Foundation, Inc.
+   Copyright (C) 88, 91, 1995-2002 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,8 +29,10 @@
 
 #include "system.h"
 #include "closeout.h"
+#include "dirname.h"
 #include "error.h"
 #include "full-write.h"
+#include "posixver.h"
 #include "safe-read.h"
 #include "xstrtol.h"
 
@@ -39,18 +41,23 @@
 
 #define AUTHORS N_ ("Torbjorn Granlund and Richard M. Stallman")
 
+#define DEFAULT_SUFFIX_LENGTH 2
+
 /* The name this program was run with. */
 char *program_name;
 
 /* Base name of output files.  */
+static char const *outbase;
+
+/* Name of output files.  */
 static char *outfile;
 
 /* Pointer to the end of the prefix in OUTFILE.
    Suffixes are inserted here.  */
 static char *outfile_mid;
 
-/* Pointer to the end of OUTFILE. */
-static char *outfile_end;
+/* Length of OUTFILE's suffix.  */
+static size_t suffix_length = DEFAULT_SUFFIX_LENGTH;
 
 /* Name of input file.  May be "-".  */
 static char *infile;
@@ -70,6 +77,7 @@ static struct option const longopts[] =
   {"bytes", required_argument, NULL, 'b'},
   {"lines", required_argument, NULL, 'l'},
   {"line-bytes", required_argument, NULL, 'C'},
+  {"suffix-length", required_argument, NULL, 'a'},
   {"verbose", no_argument, NULL, 2},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
@@ -92,54 +100,74 @@ Usage: %s [OPTION] [INPUT [PREFIX]]\n\
 Output fixed-size pieces of INPUT to PREFIXaa, PREFIXab, ...; default\n\
 PREFIX is `x'.  With no INPUT, or when INPUT is -, read standard input.\n\
 \n\
+"), stdout);
+      fputs (_("\
 Mandatory arguments to long options are mandatory for short options too.\n\
+"), stdout);
+      fprintf (stdout, _("\
+  -a, --suffix-length=N   use suffixes of length N (default %d)\n\
   -b, --bytes=SIZE        put SIZE bytes per output file\n\
   -C, --line-bytes=SIZE   put at most SIZE bytes of lines per output file\n\
   -l, --lines=NUMBER      put NUMBER lines per output file\n\
-"), stdout);
+"), DEFAULT_SUFFIX_LENGTH);
       fputs (_("\
-  -NUMBER                 same as -l NUMBER\n\
       --verbose           print a diagnostic to standard error just\n\
                             before each output file is opened\n\
-      --help              display this help and exit\n\
-      --version           output version information and exit\n\
+"), stdout);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      fputs (_("\
 \n\
 SIZE may have a multiplier suffix: b for 512, k for 1K, m for 1 Meg.\n\
 "), stdout);
-      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
-/* Compute the next sequential output file name suffix and store it
-   into the string `outfile' at the position pointed to by `outfile_mid'.  */
+/* Compute the next sequential output file name and store it into the
+   string `outfile'.  */
 
 static void
 next_file_name (void)
 {
-  static unsigned n_digits = 2;
-  char *p;
-
-  /* Change any suffix of `z's to `a's.  */
-  for (p = outfile_end - 1; *p == 'z'; p--)
+  if (! outfile)
     {
-      *p = 'a';
+      /* Allocate and initialize the first file name.  */
+
+      size_t outbase_length = strlen (outbase);
+      size_t outfile_length = outbase_length + suffix_length;
+      if (outfile_length + 1 < outbase_length)
+	xalloc_die ();
+      outfile = xmalloc (outfile_length + 1);
+      outfile_mid = outfile + outbase_length;
+      memcpy (outfile, outbase, outbase_length);
+      memset (outfile_mid, 'a', suffix_length);
+      outfile[outfile_length] = 0;
+
+#if ! _POSIX_NO_TRUNC && HAVE_PATHCONF && defined _PC_NAME_MAX
+      /* POSIX requires that if the output file name is too long for
+	 its directory, `split' must fail without creating any files.
+	 This must be checked for explicitly on operating systems that
+	 silently truncate file names.  */
+      {
+	char *dir = dir_name (outfile);
+	long name_max = pathconf (dir, _PC_NAME_MAX);
+	if (0 <= name_max && name_max < base_len (base_name (outfile)))
+	  error (EXIT_FAILURE, ENAMETOOLONG, "%s", outfile);
+	free (dir);
+      }
+#endif
     }
-
-  /* Increment the rightmost non-`z' character that was present before the
-     above z/a substitutions.  There is guaranteed to be such a character.  */
-  ++(*p);
-
-  /* If the result of that increment operation yielded a `z' and there
-     are only `z's to the left of it, then append two more `a' characters
-     to the end and add 1 (-1 + 2) to the number of digits (we're taking
-     out this `z' and adding two `a's).  */
-  if (*p == 'z' && p == outfile_mid)
+  else
     {
-      ++n_digits;
-      ++outfile_mid;
-      *outfile_end++ = 'a';
-      *outfile_end++ = 'a';
+      /* Increment the suffix in place, if possible.  */
+
+      char *p;
+      for (p = outfile_mid + suffix_length; outfile_mid < p; *--p = 'a')
+	if (p[-1]++ != 'z')
+	  return;
+      error (EXIT_FAILURE, 0, _("Output file suffixes exhausted"));
     }
 }
 
@@ -342,7 +370,6 @@ main (int argc, char **argv)
   int in_blk_size;		/* optimal block size of input file device */
   char *buf;			/* file i/o buffer */
   int accum = 0;
-  char *outbase;
   int c;
   int digits_optind = 0;
 
@@ -364,8 +391,8 @@ main (int argc, char **argv)
       int this_optind = optind ? optind : 1;
       long int tmp_long;
 
-      c = getopt_long (argc, argv, "0123456789vb:l:C:", longopts, (int *) 0);
-      if (c == EOF)
+      c = getopt_long (argc, argv, "0123456789C:a:b:l:", longopts, NULL);
+      if (c == -1)
 	break;
 
       switch (c)
@@ -373,6 +400,16 @@ main (int argc, char **argv)
 	case 0:
 	  break;
 
+	case 'a':
+	  if (xstrtol (optarg, NULL, 10, &tmp_long, "") != LONGINT_OK
+	      || tmp_long < 0 || tmp_long > SIZE_MAX)
+	    {
+	      error (0, 0, _("%s: invalid suffix length"), optarg);
+	      usage (EXIT_FAILURE);
+	    }
+	  suffix_length = tmp_long;
+	  break;
+
 	case 'b':
 	  if (split_type != type_undef)
 	    {
@@ -457,6 +494,12 @@ main (int argc, char **argv)
 	}
     }
 
+  if (digits_optind && 200112 <= posix2_version ())
+    {
+      error (0, 0, _("`-%d' option is obsolete; use `-l %d'"), accum, accum);
+      usage (EXIT_FAILURE);
+    }
+
   /* Handle default case.  */
   if (split_type == type_undef)
     {
@@ -500,17 +543,6 @@ main (int argc, char **argv)
   /* No output file is open now.  */
   output_desc = -1;
 
-  /* Copy the output file prefix so we can add suffixes to it.
-     26**29 is certainly enough output files!  */
-
-  outfile = xmalloc (strlen (outbase) + 30);
-  strcpy (outfile, outbase);
-  outfile_mid = outfile + strlen (outfile);
-  outfile_end = outfile_mid + 2;
-  memset (outfile_mid, 0, 30);
-  outfile_mid[0] = 'a';
-  outfile_mid[1] = 'a' - 1;  /* first call to next_file_name makes it an 'a' */
-
   /* Get the optimal block size of input device and make a buffer.  */
 
   if (fstat (input_desc, &stat_buf) < 0)
