@@ -1,5 +1,5 @@
 /* sum -- checksum and count the blocks in a file
-   Copyright (C) 86, 89, 91, 1995-1999 Free Software Foundation, Inc.
+   Copyright (C) 86, 89, 91, 1995-2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Like BSD sum or SysV sum -r, except like SysV sum if -s option is given. */
 
@@ -25,13 +25,15 @@
 #include <sys/types.h>
 #include <getopt.h>
 #include "system.h"
+#include "closeout.h"
 #include "error.h"
+#include "human.h"
 #include "safe-read.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "sum"
 
-#define AUTHORS "Kayvan Aghaiepour and David MacKenzie"
+#define AUTHORS N_ ("Kayvan Aghaiepour and David MacKenzie")
 
 /* The name this program was run with. */
 char *program_name;
@@ -39,9 +41,6 @@ char *program_name;
 /* Nonzero if any of the files read were the standard input. */
 static int have_read_stdin;
 
-/* Right-rotate 32-bit integer variable C. */
-#define ROTATE_RIGHT(c) if ((c) & 01) (c) = ((c) >>1) + 0x8000; else (c) >>= 1;
-
 static struct option const longopts[] =
 {
   {"sysv", no_argument, NULL, 's'},
@@ -62,17 +61,19 @@ usage (int status)
 Usage: %s [OPTION]... [FILE]...\n\
 "),
 	      program_name);
-      printf (_("\
+      fputs (_("\
 Print checksum and block counts for each FILE.\n\
 \n\
   -r              defeat -s, use BSD sum algorithm, use 1K blocks\n\
   -s, --sysv      use System V sum algorithm, use 512 bytes blocks\n\
-      --help      display this help and exit\n\
-      --version   output version information and exit\n\
+"), stdout);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      fputs (_("\
 \n\
 With no FILE, or when FILE is -, read standard input.\n\
-"));
-      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
+"), stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -87,9 +88,10 @@ static int
 bsd_sum_file (const char *file, int print_name)
 {
   register FILE *fp;
-  register unsigned long checksum = 0; /* The checksum mod 2^16. */
-  register long total_bytes = 0; /* The number of bytes. */
+  register int checksum = 0;	/* The checksum mod 2^16. */
+  register uintmax_t total_bytes = 0;	/* The number of bytes. */
   register int ch;		/* Each character read. */
+  char hbuf[LONGEST_HUMAN_READABLE + 1];
 
   if (STREQ (file, "-"))
     {
@@ -111,7 +113,7 @@ bsd_sum_file (const char *file, int print_name)
   while ((ch = getc (fp)) != EOF)
     {
       total_bytes++;
-      ROTATE_RIGHT (checksum);
+      checksum = (checksum >> 1) + ((checksum & 1) << 15);
       checksum += ch;
       checksum &= 0xffff;	/* Keep it within bounds. */
     }
@@ -130,7 +132,8 @@ bsd_sum_file (const char *file, int print_name)
       return -1;
     }
 
-  printf ("%05lu %5ld", checksum, (total_bytes + 1024 - 1) / 1024);
+  printf ("%05d %5s", checksum,
+	  human_readable_inexact (total_bytes, hbuf, 1, 1024, human_ceiling));
   if (print_name > 1)
     printf (" %s", file);
   putchar ('\n');
@@ -149,8 +152,13 @@ sysv_sum_file (const char *file, int print_name)
   int fd;
   unsigned char buf[8192];
   register int bytes_read;
-  register unsigned long checksum = 0;
-  long total_bytes = 0;
+  uintmax_t total_bytes = 0;
+  char hbuf[LONGEST_HUMAN_READABLE + 1];
+  int r;
+  int checksum;
+
+  /* The sum of all the input bytes, modulo (UINT_MAX + 1).  */
+  register unsigned int s = 0;
 
   if (STREQ (file, "-"))
     {
@@ -174,7 +182,7 @@ sysv_sum_file (const char *file, int print_name)
       register int i;
 
       for (i = 0; i < bytes_read; i++)
-	checksum += buf[i];
+	s += buf[i];
       total_bytes += bytes_read;
     }
 
@@ -192,7 +200,11 @@ sysv_sum_file (const char *file, int print_name)
       return -1;
     }
 
-  printf ("%lu %ld", checksum % 0xffff, (total_bytes + 512 - 1) / 512);
+  r = (s & 0xffff) + ((s & 0xffffffff) >> 16);
+  checksum = (r & 0xffff) + (r >> 16);
+
+  printf ("%d %s", checksum,
+	  human_readable_inexact (total_bytes, hbuf, 1, 512, human_ceiling));
   if (print_name)
     printf (" %s", file);
   putchar ('\n');
@@ -206,13 +218,15 @@ main (int argc, char **argv)
   int errors = 0;
   int optc;
   int files_given;
-  int (*sum_func) () = bsd_sum_file;
+  int (*sum_func) PARAMS ((const char *, int)) = bsd_sum_file;
 
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+  atexit (close_stdout);
+
   have_read_stdin = 0;
 
   while ((optc = getopt_long (argc, argv, "rs", longopts, NULL)) != -1)
