@@ -1,5 +1,5 @@
 /* tac - concatenate and print files in reverse
-   Copyright (C) 1988-1991, 1995-2001 Free Software Foundation, Inc.
+   Copyright (C) 1988-1991, 1995-2002 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -132,16 +132,18 @@ Usage: %s [OPTION]... [FILE]...\n\
 Write each FILE to standard output, last line first.\n\
 With no FILE, or when FILE is -, read standard input.\n\
 \n\
-Mandatory arguments to long options are mandatory for short options too.\n\
-  -b, --before             attach the separator before instead of after\n\
 "), stdout);
       fputs (_("\
+Mandatory arguments to long options are mandatory for short options too.\n\
+"), stdout);
+      fputs (_("\
+  -b, --before             attach the separator before instead of after\n\
   -r, --regex              interpret the separator as a regular expression\n\
   -s, --separator=STRING   use STRING as the separator instead of newline\n\
-      --help               display this help and exit\n\
-      --version            output version information and exit\n\
 "), stdout);
-      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -451,15 +453,13 @@ save_stdin (FILE **g_tmp, char **g_tempfile)
       if (bytes_read < 0)
 	error (EXIT_FAILURE, errno, _("stdin: read error"));
 
-      /* Don't bother checking for failure inside the loop -- check after.  */
-      fwrite (G_buffer, 1, bytes_read, tmp);
+      if (fwrite (G_buffer, 1, bytes_read, tmp) != bytes_read)
+	break;
     }
 
   if (ferror (tmp) || fflush (tmp) == EOF)
     error (EXIT_FAILURE, errno, "%s", tempfile);
 
-  rewind (tmp);
-
   SET_BINARY (fileno (tmp));
   *g_tmp = tmp;
   *g_tempfile = tempfile;
