@@ -99,10 +99,10 @@ Usage: %s [OPTION] [FILE]\n\
 Write totally ordered list consistent with the partial ordering in FILE.\n\
 With no FILE, or when FILE is -, read standard input.\n\
 \n\
-      --help       display this help and exit\n\
-      --version    output version information and exit\n"),
-	      program_name);
-      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
+"), program_name);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
 
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
