@@ -1,5 +1,5 @@
 /* unexpand - convert spaces to tabs
-   Copyright (C) 89, 91, 1995-2001 Free Software Foundation, Inc.
+   Copyright (C) 89, 91, 1995-2002 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -42,8 +42,8 @@
 #include <sys/types.h>
 #include "system.h"
 #include "closeout.h"
-
 #include "error.h"
+#include "posixver.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "unexpand"
@@ -381,21 +381,17 @@ Convert spaces in each FILE to tabs, writing to standard output.\n\
 With no FILE, or when FILE is -, read standard input.\n\
 \n\
 "), stdout);
-     fputs (_("\
+      fputs (_("\
 Mandatory arguments to long options are mandatory for short options too.\n\
 "), stdout);
-     fputs (_("\
+      fputs (_("\
   -a, --all           convert all whitespace, instead of initial whitespace\n\
   -t, --tabs=NUMBER   have tabs NUMBER characters apart instead of 8\n\
   -t, --tabs=LIST     use comma separated list of explicit tab positions\n\
 "), stdout);
-     fputs (HELP_OPTION_DESCRIPTION, stdout);
-     fputs (VERSION_OPTION_DESCRIPTION, stdout);
-     fputs (_("\
-\n\
-Instead of -t NUMBER or -t LIST, -NUMBER or -LIST may be used.\n\
-"), stdout);
-      puts (_("\nReport bugs to <bug-textutils@gnu.org>."));
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
@@ -410,6 +406,8 @@ main (int argc, char **argv)
      so that only leading white space will be considered.  */
   int convert_first_only = 0;
 
+  bool obsolete_tablist = false;
+
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -423,7 +421,8 @@ main (int argc, char **argv)
   tab_list = NULL;
   first_free_tab = 0;
 
-  while ((c = getopt_long (argc, argv, "at:,0123456789", longopts, NULL)) != -1)
+  while ((c = getopt_long (argc, argv, ",0123456789at:", longopts, NULL))
+	 != -1)
     {
       switch (c)
 	{
@@ -445,6 +444,7 @@ main (int argc, char **argv)
 	case ',':
 	  add_tabstop (tabval);
 	  tabval = -1;
+	  obsolete_tablist = true;
 	  break;
 	case_GETOPT_HELP_CHAR;
 	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
@@ -452,10 +452,18 @@ main (int argc, char **argv)
 	  if (tabval == -1)
 	    tabval = 0;
 	  tabval = tabval * 10 + c - '0';
+	  obsolete_tablist = true;
 	  break;
 	}
     }
 
+  if (obsolete_tablist && 200112 <= posix2_version ())
+    {
+      error (0, 0,
+	     _("`-LIST' option is obsolete; use `--first-only -t LIST'"));
+      usage (EXIT_FAILURE);
+    }
+
   if (convert_first_only)
     convert_entire_line = 0;
 
